﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyLocomotive
{
    internal class SetLocomotivesGeneric<T>
        where T : class
    {
        /// <summary>
		/// Список объектов, которые храним
		/// </summary>
		private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetLocomotivesGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="locomotive">Добавляемый локомотив</param>
        /// <returns></returns>
        public int Insert(T locomotive)
        {
            return Insert(locomotive, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="locomotive">Добавляемый локомотив</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T locomotive, int position)
        {
            if (position < 0 || position > Count || _maxCount == Count)
            {
                return -1;
            }
            _places.Insert(position, locomotive);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= Count)
            {
                return null;
            }
            T memoryObj = _places[position];
            _places.RemoveAt(position);
            return memoryObj;
        }
        /// <summary>
		/// Получение объекта из набора по позиции
		/// </summary>
		/// <param name="position"></param>
		/// <returns></returns>
		public T this[int position]
        {
            get
            {
                if (position >= 0 && position < Count)
                {
                    return _places[position];
                }
                return null;
            }
            set
            {
                if (position >= 0 && position < Count)
                {
                    _places[position] = value;
                }
            }
        }
        /// <summary>
		/// Проход по набору до первого пустого
		/// </summary>
		/// <returns></returns>
		public IEnumerable<T> GetLocomotives()
        {
            foreach (var locomotive in _places)
            {
                if (locomotive != null)
                {
                    yield return locomotive;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
